import { observable } from 'mobx';
import { createComputed, createEventHandlers, checkAuth } from './util';
import { createWidgets, createInitData, disposeWidget } from './widget';
import mergeRenderer from './merge-renderer';
import { createDataset, EXTRA_API, createStateDataSourceVar, generateParamsParser, setConfig } from '../datasources/index';
import lodashGet from 'lodash.get';


export function createPage(
  lifecycle,
  widgetProps,
  pageState,
  pageComputed,
  evtListeners,
  dataBinds,
  app,
  $page,
  context,
  pageAttributes
) {
  const evtHandlers = createEventHandlers(evtListeners, context);

  function extractLifecyles() {
    const result = { ...lifecycle };
    const nameMaps = [
      ['onReady', 'onPageReady'],
    ];
    nameMaps.forEach(e => {
      if (!result[e[0]] && result[e[1]]) {
        result[e[0]] = result[e[1]];
        delete result[e[1]];
      }
    });
    result['onShareAppMessage'] = (res) => {
      if (res?.from === 'button' && res?.target?.dataset?.weda_share_info) {
        return res?.target?.dataset?.weda_share_info;
      }
      if (res?.from === 'menu' && pageAttributes?.appShareMessage) {
        let { enable, pageId, params, imageUrl, title } = pageAttributes.appShareMessage;
        if (enable) {
          pageId = pageId ? pageId.replace(/^(\.)?\//, '') : pageId;
          let realPath = `/pages/${pageId}/index` + (params ? '?' + params.map(pair => pair.key + '=' + pair.value).join('&') : '');
          return {
            path: realPath,
            imageUrl,
            title
          };
        }
      }
      try {
        return lifecycle?.['onShareAppMessage']?.() || {};
      } catch (error) {
        console.log(error);
        return {};
      }
    }
    return result;
  }

  return Component({
    _componentType: 'page',
    data: createInitData(widgetProps, dataBinds),
    lifetimes: {
      attached() {
        const $page = this.getWeAppInst()
        // createWidgets 从上面移到这里是为了 i18n 切换语言的时候页面能生效
        $page.widgets = {};
        const { rootWidget, widgets } = createWidgets(widgetProps, dataBinds, $page.widgets, context);
        this._rootWidget = rootWidget;
        this._widgets = widgets;
        this._pageActive = true;
        this.__handlerAttached__ = (e) => {
          if (!$page.componentMethods) {
            $page.componentMethods = {};
          }
          const [id, index = 0] = e.target.id.split('-');
          if (!$page.componentMethods[id]) {
            $page.componentMethods[id] = [];
          }
          $page.componentMethods[id][index] = e.detail.methods;
        };
        this.__handlerDetached__ = (e) => {
          const [id, index = 0] = e.target.id.split('-');
          if ($page.componentMethods) {
            const components = $page.componentMethods[id];
            if (components) {
              components[index] = undefined;
              if (components.filter(com => com !== undefined).length === 0) {
                delete $page.componentMethods[id];
              }
            }
          }
        }
        this._disposers = this.initMergeRenderer(widgets);
      },
      detached() {
        disposeWidget(this._rootWidget);
      },
    },
    pageLifetimes: {
      // 组件所在页面的生命周期函数，定义下给运营平台上报用
      show: function () {
      },
      hide: function () {
      },
    },
    methods: {
      _pageActive: true,
      /** page lifecycles **/
      ...extractLifecyles(),
      ...evtHandlers,
      ...mergeRenderer,
      onLoad(options) {
        const $page = this.getWeAppInst()

        app.__internal__.activePage = $page;
        setConfig({ currentPageId: $page.uuid });
        this._pageActive = true;

        let query = decodePageQuery(options || {});
        EXTRA_API.setParams($page.uuid, query);
        createStateDataSourceVar($page.uuid, generateParamsParser({ app, $page }));

        const hook = lifecycle.onLoad || lifecycle.onPageLoad;
        hook && hook.call(this, query);
      },
      onUnload() {
        const $page = this.getWeAppInst()
        this._disposers.forEach(dispose => dispose());

        const hook = lifecycle.onUnload || lifecycle.onPageUnload;
        hook && hook.call(this);
      },
      async onShow() {
        const $page = this.getWeAppInst()
        app.__internal__.activePage = $page;
        setConfig({ currentPageId: $page.uuid });
        $page.widgets = this._widgets;
        this._pageActive = true;

        const hook = lifecycle.onShow || lifecycle.onPageShow;
        hook && hook.call(this);

        // 权限检查
        if (await checkAuth(app, app.id, $page)) {
          this.setData({
            weDaHasLogin: true,
          });
        }
      },
      onHide() {
        const $page = this.getWeAppInst()
        const hook = lifecycle.onHide || lifecycle.onPageHide;
        hook && hook.call(this);
        this._pageActive = false;
      },

      getWeAppInst() {
        if (!this.$WEAPPS_PAGE) {
          $page.state = observable(pageState);
          let dataset = createDataset($page.uuid);
          $page.dataset = dataset;
          $page.state.dataset = dataset;
          $page.computed = createComputed(pageComputed);
          $page.setState = (userSetState) => {
            Object.keys(userSetState).forEach((keyPath) => {
              app.utils.set($page.dataset.state, keyPath, userSetState[keyPath]);
            });
          };
          $page.invokeComponentMethod = ({ component, method, params }) => {
            const components = lodashGet($page.componentMethods, component);
            let componentInstance = undefined
            if (Array.isArray(components)) {
              if(components.length > 1){
                throw new Error('调用方法失败：id为'+component+'的组件拥有多个实例')
              }
              componentInstance = components[0]
              // components.forEach(component => {
              //   component[method](params);
              // })
            } else {
              componentInstance = components
            }

            if(!componentInstance?.[method] || typeof componentInstance[method]!= 'function') {
              throw new Error('调用方法失败：未找到id为'+component+'下的方法'+method)
            }
            return componentInstance[method](params);
          };
          this.$WEAPPS_PAGE = $page;
        }
        return this.$WEAPPS_PAGE
      },
    },
  });
}

function decodePageQuery(query) {
  return Object.keys(query)
    .reduce((decoded, key) => {
      decoded[key] = decodeURIComponent(query[key]);
      return decoded;
    }, {});
}
